@@ -18,7 +18,7 @@ from subprocess import Popen, PIPE
 from . import __version__
 from .compress import Compressor
 from .constants import *  # NOQA
-from .helpers import Error, IntegrityError
+from .helpers import Error, ErrorWithTraceback, IntegrityError
 from .helpers import bin_to_hex
 from .helpers import get_base_dir
 from .helpers import get_limited_unpacker
@@ -27,6 +27,7 @@ from .helpers import sysinfo
 from .helpers import format_file_size
 from .helpers import safe_unlink
 from .helpers import prepare_subprocess_env, ignore_sigint
+from .locking import LockTimeout, NotLocked, NotMyLock, LockFailed
 from .logger import create_logger, setup_logging
 from .helpers import msgpack
 from .repository import Repository
@@ -66,26 +67,32 @@ def os_write(fd, data):
 
 class ConnectionClosed(Error):
     """Connection closed by remote host"""
+    exit_mcode = 80
 
 
 class ConnectionClosedWithHint(ConnectionClosed):
     """Connection closed by remote host. {}"""
+    exit_mcode = 81
 
 
 class PathNotAllowed(Error):
     """Repository path not allowed: {}"""
+    exit_mcode = 83
 
 
 class InvalidRPCMethod(Error):
     """RPC method {} is not valid"""
+    exit_mcode = 82
 
 
 class UnexpectedRPCDataFormatFromClient(Error):
     """Borg {}: Got unexpected RPC data format from client."""
+    exit_mcode = 85
 
 
 class UnexpectedRPCDataFormatFromServer(Error):
     """Got unexpected RPC data format from server:\n{}"""
+    exit_mcode = 86
 
     def __init__(self, data):
         try:
@@ -97,6 +104,11 @@ class UnexpectedRPCDataFormatFromServer(Error):
         super().__init__(data)
 
 
+class ConnectionBrokenWithHint(Error):
+    """Connection to remote host is broken. {}"""
+    exit_mcode = 87
+
+
 # Protocol compatibility:
 # In general the server is responsible for rejecting too old clients and the client it responsible for rejecting
 # too old servers. This ensures that the knowledge what is compatible is always held by the newer component.
@@ -239,6 +251,19 @@ class RepositoryServer:  # pragma: no cover
                         args = self.filter_args(f, args)
                         res = f(**args)
                     except BaseException as e:
+                        # These exceptions are reconstructed on the client end in RemoteRepository.call_many(),
+                        # and will be handled just like locally raised exceptions. Suppress the remote traceback
+                        # for these, except ErrorWithTraceback, which should always display a traceback.
+                        reconstructed_exceptions = (
+                            Repository.InvalidRepository,
+                            Repository.InvalidRepositoryConfig,
+                            Repository.DoesNotExist,
+                            Repository.AlreadyExists,
+                            Repository.PathAlreadyExists,
+                            PathNotAllowed,
+                            Repository.InsufficientFreeSpaceError,
+                            Repository.StorageQuotaExceeded,
+                        )
                         if dictFormat:
                             ex_short = traceback.format_exception_only(e.__class__, e)
                             ex_full = traceback.format_exception(*sys.exc_info())
@@ -246,12 +271,7 @@ class RepositoryServer:  # pragma: no cover
                             if isinstance(e, Error):
                                 ex_short = [e.get_message()]
                                 ex_trace = e.traceback
-                            if isinstance(e, (Repository.DoesNotExist, Repository.AlreadyExists, PathNotAllowed)):
-                                # These exceptions are reconstructed on the client end in RemoteRepository.call_many(),
-                                # and will be handled just like locally raised exceptions. Suppress the remote traceback
-                                # for these, except ErrorWithTraceback, which should always display a traceback.
-                                pass
-                            else:
+                            if not isinstance(e, reconstructed_exceptions):
                                 logging.debug('\n'.join(ex_full))
 
                             try:
@@ -274,12 +294,7 @@ class RepositoryServer:  # pragma: no cover
 
                             os_write(stdout_fd, msg)
                         else:
-                            if isinstance(e, (Repository.DoesNotExist, Repository.AlreadyExists, PathNotAllowed)):
-                                # These exceptions are reconstructed on the client end in RemoteRepository.call_many(),
-                                # and will be handled just like locally raised exceptions. Suppress the remote traceback
-                                # for these, except ErrorWithTraceback, which should always display a traceback.
-                                pass
-                            else:
+                            if not isinstance(e, reconstructed_exceptions):
                                 if isinstance(e, Error):
                                     tb_log_level = logging.ERROR if e.traceback else logging.DEBUG
                                     msg = e.get_message()
@@ -414,7 +429,10 @@ class SleepingBandwidthLimiter:
                 self.ratelimit_last = time.monotonic()
             if len(to_send) > self.ratelimit_quota:
                 to_send = to_send[:self.ratelimit_quota]
-        written = os.write(fd, to_send)
+        try:
+            written = os.write(fd, to_send)
+        except BrokenPipeError:
+            raise ConnectionBrokenWithHint("Broken Pipe") from None
         if self.ratelimit:
             self.ratelimit_quota -= written
         return written
@@ -517,6 +535,7 @@ class RemoteRepository:
 
     class RPCServerOutdated(Error):
         """Borg server is too old for {}. Required version {}"""
+        exit_mcode = 84
 
         @property
         def method(self):
@@ -739,7 +758,13 @@ This problem will go away as soon as the server has been upgraded to 1.0.7+.
             old_server = b'exception_args' not in unpacked
             args = unpacked.get(b'exception_args')
 
-            if error == 'DoesNotExist':
+            if error == 'Error':
+                raise Error(args[0].decode())
+            elif error == 'ErrorWithTraceback':
+                raise ErrorWithTraceback(args[0].decode())
+            elif error == 'InvalidRepository':
+                raise Repository.InvalidRepository(self.location.processed)
+            elif error == 'DoesNotExist':
                 raise Repository.DoesNotExist(self.location.processed)
             elif error == 'AlreadyExists':
                 raise Repository.AlreadyExists(self.location.processed)
@@ -760,6 +785,10 @@ This problem will go away as soon as the server has been upgraded to 1.0.7+.
                     raise PathNotAllowed('(unknown)')
                 else:
                     raise PathNotAllowed(args[0].decode())
+            elif error == 'PathPermissionDenied':
+                raise Repository.PathPermissionDenied(args[0].decode())
+            elif error == 'PathAlreadyExists':
+                raise Repository.PathAlreadyExists(args[0].decode())
             elif error == 'ParentPathDoesNotExist':
                 raise Repository.ParentPathDoesNotExist(args[0].decode())
             elif error == 'ObjectNotFound':
@@ -772,6 +801,32 @@ This problem will go away as soon as the server has been upgraded to 1.0.7+.
                     raise InvalidRPCMethod('(not available)')
                 else:
                     raise InvalidRPCMethod(args[0].decode())
+            elif error == 'LockTimeout':
+                if old_server:
+                    raise LockTimeout('(not available)')
+                else:
+                    raise LockTimeout(args[0].decode())
+            elif error == 'LockFailed':
+                if old_server:
+                    raise LockFailed('(not available)', '')
+                else:
+                    raise LockFailed(args[0].decode(), args[1].decode())
+            elif error == 'NotLocked':
+                if old_server:
+                    raise NotLocked('(not available)')
+                else:
+                    raise NotLocked(args[0].decode())
+            elif error == 'NotMyLock':
+                if old_server:
+                    raise NotMyLock('(not available)')
+                else:
+                    raise NotMyLock(args[0].decode())
+            elif error == 'InsufficientFreeSpaceError':
+                raise Repository.InsufficientFreeSpaceError(args[0].decode(), args[1].decode())
+            elif error == 'InvalidRepositoryConfig':
+                raise Repository.InvalidRepositoryConfig(self.location.processed, args[1].decode())
+            elif error == 'StorageQuotaExceeded':
+                raise Repository.StorageQuotaExceeded(args[0].decode(), args[1].decode())
             else:
                 raise self.RPCError(unpacked)
 
